% Core part of learning with a Rectified Gaussian Belief Network
%
% This script:
%    Picks a data vector at random and clamps it
%    Gibbs samples "reps" times, collecting weight changes
%    Changes the weights
%    
% Changed to set x1 and x2 to zero at the beginning of each Gibbs sampling run (ZG 3/5/97)
% Changed to use every Gibbs sample after 4th for learning (ZG 3/5/97)

% pick a data vector at random and clamp input

d=floor(rand*N)+1;   
x3=Data(d,:)';
x1=zeros(n1,1);
x2=zeros(n2,1);

% set change in wts to zero 

dW12=zeros(n2,n1);   
dW23=zeros(n3,n2);
db1=zeros(n1,1);
db2=zeros(n2,1);
db3=zeros(n3,1);
dlv1=0;           % note: variance changed in log domain 
dlv2=0;           
dlv3=0;           

% Gibbs sample reps times

etafrac=1/(reps-4);

for i=1:reps,
  [x1,x2,x3]=sample(x1,x2,x3,W12,W23,b1,b2,b3,v1,v2,v3,NL,1);
  
  % collect weight changes
  
  % if rem(i,2)==0,  % only for every other Gibbs sample
  if  i>4,

    delx1=(x1-b1);
    db1=db1+eta2*etafrac*delx1/v1;    % delta rule

    x1r=rect(x1,NL);         
    
    delx2=(x2-W12*x1r-b2);
    dW12=dW12+eta2*etafrac*delx2*x1r'/v2;
    db2=db2+eta2*etafrac*delx2/v2;
    
    x2r=rect(x2,1);          

    delx3=(x3-W23*x2r-b3);
    dW23=dW23+eta3*etafrac*delx3*x2r'/v3;
    db3=db3+eta3*etafrac*delx3/v3;

    dlv1=dlv1+eta2*etafrac*(delx1'*delx1/v1-n1)/2; % log change to variance
    dlv2=dlv2+eta2*etafrac*(delx2'*delx2/v2-n2)/2; 
    dlv3=dlv3+eta3*etafrac*(delx3'*delx3/v3-n3)/2; 

  end;
  
end;

% update weights with weight decay

b1=b1+db1-eta2*lambda*b1;
W12=W12+dW12-eta2*lambda*W12;
v1=exp(log(v1)+dlv1-eta2*lambdav*log(v1));

b2=b2+db2-eta2*lambda*b2;
W23=W23+dW23-eta3*lambda*W23;
v2=exp(log(v2)+dlv2-eta2*lambdav*log(v2));

b3=b3+db3-eta3*lambda*b3;
v3=exp(log(v3)+dlv3-eta3*lambdav*log(v3));

activ(d,:)=[x1' x2'];

