% function [x1n,x2n,x3n]=sample(x1,x2,x3,W12,W23,b1,b2,b3,v1,v2,v3,NL,flag);
% 
% a single pass of Gibbs sampling from a three layer 
% rectified Gaussian belief network
%
% if flag=1 the input layer (x3) is clamped
% if flag=0 the input layer (x3) is unclamped
% if flag is a vector,  then it indicates which elements of x3 are clamped.
%         flag must be of length x3.

function [x1,x2,x3]=sample(x1,x2,x3,W12,W23,b1,b2,b3,v1,v2,v3,NL,flag);

n3=length(x3);

%%% sample from the top layer

x1=topsample(x1,x2,W12,b1,b2,v1,v2,NL);

%%% sample from middle layer given top and bottom layers

x2=midsample(x1,x2,x3,W12,W23,b2,b3,v1,v2,v3,NL);

%%% sample from bottom layer given middle layer and clamped values

if flag==0,
  x2r=rect(x2,1);
  x3=W23*x2r+b3+randn(n3,1)*sqrt(v3);
elseif (size(flag)==[n3 1]),
  x2r=rect(x2,1);
  x3=x3.*flag+(W23*x2r+b3+randn(n3,1)*sqrt(v3)).*(~flag);
elseif flag==1,
else
  disp('Error in sample: flag must be either a scalar indicator or a vector of length n3');
end;

  
  