
function samp=threegauss(mun,vn,mup,vp);

  tiny=exp(-700);

  % probability of 0 under negative and positive Gaussian 
  % has to be equal, from this we compute mixing proportions
  
  pneg0=1/sqrt(2*pi*vn)*exp(-(mun^2)/(2*vn));
  ppos0=1/sqrt(2*pi*vp)*exp(-(mup^2)/(2*vp));

  ppos1=1/sqrt(2*pi*vp)*exp(-((mup-1)^2)/(2*vp));
  pneg1=1/sqrt(2*pi*vn)*exp(-((mun-1)^2)/(2*vn));

  alpha=[1 pneg0/(ppos0+tiny) ppos1/(pneg1+tiny)*pneg0/(ppos0+tiny)]+tiny;
  alpha=alpha/sum(alpha);

  % sample from mixture once
  
  switch=rand;  
  if switch<alpha(1),
    sneg=mun+randn*sqrt(vn);
    if sneg<=0,
      samp=sneg;
      return;
    end;
  elseif switch<alpha(1)+alpha(2),
    spos=mup+randn*sqrt(vp);
    if spos>0 & spos<1,
      samp=spos;
      return;
    end;
  else
    sneg=mun+randn*sqrt(vn);
    if sneg>1,
      samp=sneg;
      return;
    end;
  end;    
  
  P=[alpha(1)*cumnorm(mun,vn) alpha(2)*(cumnorm(-mup,vp)-cumnorm(1-mup,vp)) alpha(3)*cumnorm(1-mun,vn)];  
  
  inttot=sum(P);
  if inttot==0,
    samp=0;
    return;
  end;
  P=P/inttot;
    
  % sample
  
  zn=mun/sqrt(vn);
  zp=mup/sqrt(vp);
  zp1=(mup-1)/sqrt(vp);
  zs=(mun-1)/sqrt(vn);
  
  r=rand;
    
    if r<P(1),                             % sample from negative portion 
      if zn>1,    % use tail sampling
	samp=-samptail(-mun,vn);
      else        % use simple rejection sampling 
	samp=-samprej(-mun,vn);
      end;
    elseif r>=P(1) & r<(P(1)+P(2)),          % sample from middle portion
      done=0;
      while ~done,
	if zp<-1,
	  samp=samptail(mup,vp);
	elseif zp1<1,
	  samp=samprej(mup,vp);
	else 
	  samp=1-samptail(1-mup,vp);
	end;
	if samp>0 & samp<1,
	  done=1;
	end
      end;
    elseif r>=P(1)+P(2),                   % sample from saturated portion
      if zs<-1,
	samp=1+samptail(mun-1,vn);
      else        % use simple rejection sampling 
	samp=1+samprej(mun-1,vn);
      end;
    end;

end;
  